//  Module Name:    TGCS Mumford SDK Sample App
//
//  Description:	TGCS Mumford SDK Sample App
//
//  Copyright:       Toshiba Confidential
//
//                   OCO Source Materials
//
//                   Copyright (C) 2013 Toshiba Global Commerce Solutions, Inc.
//
//                   The source code for this program is not published
//                   or otherwise divested of its trade secrets,
//                   irrespective of what has been deposited with the
//                   U. S. Copyright Office.
//
//  Initial Author:	nthiang
//
//  Function: 		This demonstrates the use of the Mumford SDK. 
//
//
//  Notes:           None
//

package com.toshiba.tgcsmpossample;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;

import com.toshiba.tgcsapi.*;

import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Environment;
import android.provider.MediaStore;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager.NameNotFoundException;
import android.content.res.AssetManager;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.util.Log;
import android.view.Menu;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.Toast;

public class MainActivity extends Activity {
	
	private EditText etTarget;
	private ListView lvActions;
	private Context ctx;
	private boolean autoCutPaper = false;
	
	// For Async calls method 2
	private int expectedOutputID = 0;
	
	private static final int REQUEST_LOADCONFIG_GETFILE = 23;
	private static final int REQUEST_LOADMICREXCEPT_GETFILE = 24;
	
	private static final int ABOUT = 1;
	private static final int EG_DEVICE_COMMON_METHODS = 2;
	private static final int EG_PRINTER_PROPERTIES = 3;
	private static final int EG_PRINTER_REC = 4;
	private static final int EG_PRINTER_REC_TRANSACTION = 5;
	private static final int EG_PRINTER_REC_ASYNC_1 = 6;
	private static final int EG_PRINTER_REC_ASYNC_2 = 7;
	private static final int EG_PRINTER_REC_LINECHARS = 8;
	private static final int EG_PRINTER_REC_CUTPAPER = 9;
	private static final int EG_PRINTER_DOC = 10;
	private static final int EG_MICR = 11;
	private static final int EG_CASHDRAWER = 12;
	private static final int EG_SHOW_CONFIGURATION = 13;
	private static final int EG_LOAD_CONFIGURATION = 14;
	private static final int EG_LOAD_MICR_EXCEPTION = 15;
	private static final int EG_PRINTER_REC_ROTATE_PRINT = 16;
	private static final int EG_PRINTER_REC_ROTATE_PRINT_ESC_SEQUENCE = 18;
	private static final int EG_PRINTER_REC_ESC_SEQUENCE = 17;
	private static final int EG_TEST_GROUND = 19;
	
	
	private ActionItem AboutItem = new ActionItem("About", ABOUT);
	private ActionItem DeviceCommonMethods = new ActionItem("Device Common Methods", EG_DEVICE_COMMON_METHODS);
	private ActionItem PrinterProperties = new ActionItem("Printer Properties", EG_PRINTER_PROPERTIES);
	private ActionItem PrinterRec = new ActionItem("Printer (Receipt) Example", EG_PRINTER_REC);
	private ActionItem PrinterRecRotatePrint = new ActionItem("Printer (Receipt) rotatePrint() Example", EG_PRINTER_REC_ROTATE_PRINT);
	private ActionItem PrinterRecRotateEscPrint = new ActionItem("Printer (Receipt) rotatePrint() with ESC Sequences Example", EG_PRINTER_REC_ROTATE_PRINT_ESC_SEQUENCE);
	private ActionItem PrinterRecEscSequencePrint = new ActionItem("Printer (Receipt) ESC Sequences Example", EG_PRINTER_REC_ESC_SEQUENCE);
	private ActionItem PrinterRecTransaction = new ActionItem("Printer (Receipt) Transaction Example", EG_PRINTER_REC_TRANSACTION);
	private ActionItem PrinterRecAsync1 = new ActionItem("Printer (Receipt) AsyncMode Example 1", EG_PRINTER_REC_ASYNC_1);
	private ActionItem PrinterRecAsync2 = new ActionItem("Printer (Receipt) AsyncMode Example 2", EG_PRINTER_REC_ASYNC_2);
	private ActionItem PrinterRecLineChars = new ActionItem("Printer (Receipt) RecLineChars Example", EG_PRINTER_REC_LINECHARS);
	private ActionItem PrinterRecCutPaper = new ActionItem("Printer (Receipt) cutPaper", EG_PRINTER_REC_CUTPAPER);
	private ActionItem PrinterDoc = new ActionItem("Printer (Document) Example", EG_PRINTER_DOC);
	private ActionItem MICR = new ActionItem("MICR Example", EG_MICR);
	private ActionItem CashDrawer = new ActionItem("Cash Drawer Example", EG_CASHDRAWER);
	private ActionItem ShowConfiguration = new ActionItem("Show Configuration", EG_SHOW_CONFIGURATION);
	private ActionItem LoadConfiguration = new ActionItem("Load Configuration", EG_LOAD_CONFIGURATION);
	private ActionItem LoadMicrException = new ActionItem("Load MICR Exception File", EG_LOAD_MICR_EXCEPTION);
	private ActionItem TestGround = new ActionItem("Test Ground", EG_TEST_GROUND);
	
	private ActionItem actions[] = {
		TestGround,
		DeviceCommonMethods,
		PrinterProperties,
		PrinterRec,
		PrinterRecRotatePrint,
		PrinterRecRotateEscPrint,
		PrinterRecEscSequencePrint,
		PrinterRecTransaction,
		PrinterRecAsync1,
		PrinterRecAsync2,
		PrinterRecLineChars,
		PrinterRecCutPaper,
		PrinterDoc,
		MICR,
		CashDrawer,
		ShowConfiguration,
		LoadConfiguration,
		LoadMicrException,
		AboutItem
	};

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		
		ctx = this;
		
		initializeAssets();
		
		initializeAPI();
		
		initElements();
	}
	
	/*
	 * Create the default assets on the disk.
	 * - Config ==> Sample configuration files
	 * - MICR ==> Sample MICR Exception files 
	 * @return: void.
	 */
	private void initializeAssets() {
		String configDir = Environment.getExternalStorageDirectory().getPath().toString()+"/tgcsmpos_sample/config/";
		String micrDir = Environment.getExternalStorageDirectory().getPath().toString()+"/tgcsmpos_sample/micr/";
		createDirectoryIfNotExists(configDir);
		createDirectoryIfNotExists(micrDir);
		
		// if assets already created, check to overwrite or not.
		// default: overwrite
		boolean overwrite = true;
		
		if (overwrite) {
			clearFilesInDirectory(configDir);
			clearFilesInDirectory(micrDir);
			
			
			AssetManager assetManager = getAssets();
			String[] files = null;
			try {
				files = assetManager.list("");
				for (String filename: files) {
					InputStream in = null;
					OutputStream out = null;
					
					Log.d("TGCSSample", filename);
					if (getFileExtension(filename).equalsIgnoreCase("efg")) {
						in = assetManager.open(filename);
						File outFile = new File(configDir, filename);
						out = new FileOutputStream(outFile);
						copyFile(in, out);
						in.close(); in = null;
						out.flush(); out.close(); out = null;
					} else if (getFileExtension(filename).equalsIgnoreCase("dat")) {
						in = assetManager.open(filename);
						File outFile = new File(micrDir, filename);
						out = new FileOutputStream(outFile);
						copyFile(in, out);
						in.close(); in = null;
						out.flush(); out.close(); out = null;
					}
				}
			} catch (IOException e) {
				
				e.printStackTrace();
			}
		}
		
	}
	
	/*
	 * Copy the files from in to out.
	 * @param: in: input stream
	 * @param: out: output stream
	 * @return: void
	 */
	private void copyFile(InputStream in, OutputStream out) throws IOException {
	    byte[] buffer = new byte[1024];
	    int read;
	    while((read = in.read(buffer)) != -1){
	      out.write(buffer, 0, read);
	    }
	}
	
	/*
	 * Obtain the file extension
	 * @param: src: source string
	 * @return: extension string
	 */
	private String getFileExtension(String src) {
		String array[] = src.split("\\.");
		return array[array.length-1];
	}
	
	/*
	 * Delete the files in the given directory
	 * @param: dir: directory to delete the files from
	 * @return: void
	 */
	private void clearFilesInDirectory(String dir) {
		File fd = new File(dir);
		
		for (File file : fd.listFiles()) {
			file.delete();
		}
	}
	
	/*
	 * Initialize the API engine. For android, the bare minimum initializing needed is to 
	 * set the LogFilePath. 
	 * @return: void
	 */
	private void initializeAPI() {
		String tmp = Environment.getExternalStorageDirectory().getPath().toString()+"/tgcsmpos_sample/";
		createDirectoryIfNotExists(tmp);

		// API initializing methods that should be called when the app loads
		
		// Logging setup
		TGCSMPOSLogging.setLogFilePath(tmp);
		
		
	}
	
	/*
	 * Create the directory if it doesn't not exist. 
	 * @param: path: absolute path to create
	 * @return: void
	 */
	private void createDirectoryIfNotExists(String path) {
		File f = new File(path);
		
		if (f != null && !f.exists()) {
			f.mkdirs();
		}
	}
	
	/*
	 * Initialize the visual elements and their reaction to user interaction. 
	 * Samples are built and displayed as a list. Each item in the list reacts to the human 
	 * tap which then invokes the sample method for that item.
	 * @return: void
	 */
	private void initElements() {
		etTarget = (EditText) findViewById(R.id.target);
		lvActions = (ListView) findViewById(R.id.actionList);
		
		lvActions.setAdapter(new ArrayAdapter<ActionItem>(this, android.R.layout.simple_list_item_1, actions));
		
		lvActions.setOnItemClickListener(new OnItemClickListener() {

			@Override
			public void onItemClick(AdapterView<?> parent, View v, final int position,
					long id) {
				
				switch(actions[position].id) {
				case EG_SHOW_CONFIGURATION:
					loadConfiguredDevices();
					break;
				case EG_LOAD_CONFIGURATION:
					Intent i = new Intent();
					i.setType("*/*");
					i.setAction(Intent.ACTION_GET_CONTENT);
					i.addCategory(Intent.CATEGORY_OPENABLE);
					startActivityForResult(Intent.createChooser(i, "Choose File"), REQUEST_LOADCONFIG_GETFILE);
					break;
				case EG_LOAD_MICR_EXCEPTION:
					Intent exceptIntent = new Intent();
					exceptIntent.setType("*/*");
					exceptIntent.setAction(Intent.ACTION_GET_CONTENT);
					startActivityForResult(Intent.createChooser(exceptIntent, "Choose File"), REQUEST_LOADMICREXCEPT_GETFILE);
					break;
				default:
					// It is good to do time-consuming calls on a separate thread
					(new AsyncTask<Integer, Void, String>() {

						ProgressDialog progress;
						
						@Override
						protected void onPreExecute() {
							progress = new ProgressDialog(ctx);
							progress.setProgressStyle(ProgressDialog.STYLE_SPINNER);
							progress.setTitle(ctx.getResources().getString(R.string.title_inprogress));
							progress.show();
							super.onPreExecute();
						}

						@Override
						protected String doInBackground(Integer... params) {
							
							if (params.length == 1) {
								String logicalDeviceName = etTarget.getText().toString();
								String res = "";
								
								switch(params[0]) {
								case EG_TEST_GROUND: res = testGround(logicalDeviceName); break;
								case EG_DEVICE_COMMON_METHODS: res = DeviceCommonMethods(logicalDeviceName); break;
							    case EG_PRINTER_PROPERTIES: res = PrinterProperties(logicalDeviceName); break;
							    case EG_PRINTER_REC: res = PrinterReceipt(logicalDeviceName); break;
							    case EG_PRINTER_REC_ROTATE_PRINT: res = PrinterReceiptRotatePrintSynchronous(logicalDeviceName); break;
							    case EG_PRINTER_REC_ROTATE_PRINT_ESC_SEQUENCE: res = PrinterReceiptRotateEscSequencePrintSynchronous(logicalDeviceName); break;
							    case EG_PRINTER_REC_ESC_SEQUENCE: res = PrinterReceiptEscSequences(logicalDeviceName); break;
							    case EG_PRINTER_REC_TRANSACTION: res = PrinterReceiptTransaction(logicalDeviceName); break;
							    case EG_PRINTER_REC_ASYNC_1: res = PrinterReceiptAsyncModeMethod1(logicalDeviceName); break;
							    case EG_PRINTER_REC_ASYNC_2: res = PrinterReceiptAsyncModeMethod2(logicalDeviceName); break;
							    case EG_PRINTER_REC_LINECHARS: res = PrinterReceiptLineChars(logicalDeviceName); break;
							    case EG_PRINTER_REC_CUTPAPER: res = PrinterReceiptCutPaper(logicalDeviceName); break;
							    case EG_PRINTER_DOC: res = PrinterDocument(logicalDeviceName); break;
							    case EG_MICR: res = micrExample(logicalDeviceName); break;
							    case EG_CASHDRAWER: res = cashDrawerExample(logicalDeviceName); break;
							    case ABOUT: res = about(); break;
								}
								
								return res;
							}
							
							return "";
						}
						
						@Override
						protected void onPostExecute(String result) {
							progress.dismiss();
							
							if (!result.isEmpty()) {
								showInformativeDialog(actions[position].title, result);
							}
							
							super.onPostExecute(result);
						}
					}).execute(actions[position].id);
					break;
				}
			}
			
		});
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		//getMenuInflater().inflate(R.menu.main, menu);
		return super.onCreateOptionsMenu(menu);
	}
	
	
	
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		Log.d("TGCSTestApp", "requestCode("+requestCode+"), resultCode("+resultCode+")");
		
		switch(requestCode) {
		case REQUEST_LOADCONFIG_GETFILE:
			if (resultCode == RESULT_OK) {
				Uri fileURI = data.getData();
				
				////////////////////////////////////
				// loadConfigurationFile Example
				// Pass the Absolute Path to the API
				////////////////////////////////////
				TGCSMPOSDevice.loadConfigurationFile(getRealPathFromURI(fileURI));
				
				loadConfiguredDevices();
			}
			break;
		case REQUEST_LOADMICREXCEPT_GETFILE:
			if (resultCode == RESULT_OK) {
				Uri uri = data.getData();
				
				////////////////////////////////////
				// loadMICRExceptionFile Example
				// Pass the Absolute Path to the API
				////////////////////////////////////
				TGCSMPOSDevice.loadMICRExceptionFile(getRealPathFromURI(uri));
			}
			break;
		}
		
		super.onActivityResult(requestCode, resultCode, data);
	}

	/*
	 * Obtain the real path from a given URI
	 * @param: contentURI
	 * @return: string result
	 */
	private String getRealPathFromURI(Uri contentURI) {
	    Cursor cursor = getContentResolver().query(contentURI, null, null, null, null);
	    if (cursor == null) { // Source is Dropbox or other similar local file path
	        return contentURI.getPath();
	    } else { 
	        cursor.moveToFirst(); 
	        int idx = cursor.getColumnIndex(MediaStore.Images.ImageColumns.DATA); 
	        return cursor.getString(idx); 
	    }
	}


	// utility class 
	private class ActionItem {
		public String title;
		public int id;
		
		public ActionItem(String title, int id) {
			this.title = title;
			this.id = id;
		}

		@Override
		public String toString() {
			return title;
		}
	}
	
	////////////////////////////////////////////////////////
	// sample code for using the TGCSMPOSAPI
	////////////////////////////////////////////////////////
	/*
	 * Show information about the Sample App and the TGCSMPOS API
	 * @return: string result.
	 */
	private String about() {
		String res = "";
		try {
			PackageInfo pInfo = getPackageManager().getPackageInfo(getPackageName(), 0);
			res = String.format(getResources().getString(R.string.template_current_version), pInfo.versionName, TGCSMPOSDevice.getVersion());
		} catch (NameNotFoundException e) {
			
			e.printStackTrace();
		}
		return res;
	}
	
	////////////////////////////////////////////////////////
	// sample code for using the TGCSMPOSAPI
	////////////////////////////////////////////////////////
	/*
	 * This method demonstrates the use of common device methods supported by the TGCSMPOS API. 
	 * This method creates a virtual instance of the device and performs some of the supported
	 * common methods, showing the result of performing the series of methods with a timestamp
	 * of how long this example took. 
	 * @param: logicalDeviceName: logical device name of the target.
	 * @effects: performs tests on device
	 * @return: String result
	 */
	private String DeviceCommonMethods(String logicalDeviceName) {
		String res = "";
		
		// instantiate the device instance
		TGCSMPOSDevice device = new TGCSMPOSDevice();
		
		// attributes
		boolean claimed = false;
		boolean deviceEnabled = false;
		boolean dataEventEnabled = false;
		int outputID = 0; 
		String physicalDeviceName = "";
		
		java.util.Date start = new java.util.Date();
		try {
			// open the device
			device.open(logicalDeviceName);
			
			// claim the device for exclusive access; 3000ms timeout
			device.claim(3000);
			
			// enable the device
			device.setDeviceEnabled(true);
			
			// Device common properties
			claimed 			= device.getClaimed();
			deviceEnabled 		= device.getDeviceEnabled();
			dataEventEnabled 	= device.getDataEventEnabled();
			outputID			= device.getOutputID();
			physicalDeviceName 	= device.getPhysicalDeviceName();
			
			// relinquish exclusive access to the device
			device.release();
			
			// close the device
			device.close();
		} catch (TGCSMPOSException mpos_e) {
			res += mpos_e.getMessage() + "\n";
			try {
				// necessary clean up
				device.close();
			} catch(Exception e1) {
				// in the event open fails, close() will throw an exception
				// ignore
			}
		} catch (RuntimeException e) {
			res += e.getMessage() + "\n";
			try {
				// necessary clean up
				device.close();
			} catch(Exception e1) {
				// in the event open fails, close() will throw an exception
				// ignore
			}
		}
		
		
		if (res.isEmpty()) {
			res += "claimed: " + claimed + "\n";
			res += "deviceEnabled: " + deviceEnabled + "\n";
			res += "dataEventEnabled: " + dataEventEnabled + "\n";
			res += "outputID: " + outputID + "\n";
			res += "physicalDeviceName: " + physicalDeviceName + "\n";
			
			res += "Time taken: "+getTimeDifference(start)+"ms";
		}
		
		return res;
	}
	
	/*
	 * This method demonstrates the use of printer specific methods supported by the TGCSMPOS API. 
	 * This method creates a virtual instance of the device and performs some of the supported
	 * printer methods, focusing on printer properties, showing the result of performing the series 
	 * of methods with a timestamp of how long this example took. 
	 * @param: logicalDeviceName: logical device name of the target.
	 * @effects: performs tests on device
	 * @return: String result
	 */
	private String PrinterProperties(String logicalDeviceName) {
		String res = "";
		
		TGCSMPOS4610Printer device = new TGCSMPOS4610Printer();
		
		// attributes
		boolean AsyncMode = false;
	    boolean CapSlpLeft90 = false;
	    boolean CapSlpPresent = false;
	    boolean CapSlpRight90 = false;
	    boolean CapSlpRotate180 = false;
	    int CharacterSet = 0;
	    String CharacterSetList = "";
	    boolean CoverOpen = false;
	    int ErrorLevel = 0;
	    boolean FlagWhenIdle = false;
	    int MapMode = 0;
	    int RecLineChars = 0;
	    String RecLineCharsList = "";
	    int RecLineHeight = 0;
	    int RecLineWidth = 0;
	    boolean RecEmpty = false;
	    int SlpLineChars = 0;
	    String SlpLineCharsList = "";
	    int SlpLineHeight = 0;
	    int SlpLineSpacing = 0;
	    boolean SlpEmpty = false;
	    int SlpPrintSide = 0;
		
		java.util.Date start = new java.util.Date();
		try {
			// open the device
			device.open(logicalDeviceName);
			
			// claim the device for exclusive access; 3000ms timeout
			device.claim(3000);
			
			// enable the device
			device.setDeviceEnabled(true);
			
			// Device common properties
			AsyncMode = device.getAsyncMode();
	        CapSlpLeft90 = device.getCapSlpLeft90();
	        CapSlpPresent = device.getCapSlpPresent();
	        CapSlpRight90 = device.getCapSlpRight90();
	        CapSlpRotate180 = device.getCapSlpRotate180();
	        CharacterSet = device.getCharacterSet();
	        CharacterSetList = device.getCharacterSetList();
	        CoverOpen = device.getCoverOpen();
	        ErrorLevel = device.getErrorLevel();
	        FlagWhenIdle = device.getFlagWhenIdle();
	        MapMode = device.getMapMode();
	        RecLineChars = device.getRecLineChars();
	        RecLineCharsList = device.getRecLineCharsList();
	        RecLineHeight = device.getRecLineHeight();
	        RecLineWidth = device.getRecLineWidth();
	        RecEmpty = device.getRecEmpty();
	        SlpLineChars = device.getSlpLineChars();
	        SlpLineCharsList = device.getSlpLineCharsList();
	        SlpLineHeight = device.getSlpLineHeight();
	        SlpLineSpacing = device.getSlpLineSpacing();
	        SlpEmpty = device.getSlpEmpty();
	        SlpPrintSide = device.getSlpPrintSide();
			
			// relinquish exclusive access to the device
			device.release();
			
			// close the device
			device.close();
		} catch (TGCSMPOSException mpos_e) {
			res += mpos_e.getMessage() + "\n";
			try {
				// necessary clean up
				device.close();
			} catch(Exception e1) {
				// in the event open fails, close() will throw an exception
				// ignore
			}
		} catch (RuntimeException e) {
			res += e.getMessage() + "\n";
			try {
				// necessary clean up
				device.close();
			} catch(Exception e1) {
				// in the event open fails, close() will throw an exception
				// ignore
			}
		}
		
		
		if (res.isEmpty()) {
		
			res += "AsyncMode = "+AsyncMode+"\n";
		    res += "CapSlpLeft90 = "+CapSlpLeft90+"\n";
		    res += "CapSlpPresent = "+CapSlpPresent+"\n";
		    res += "CapSlpRight90 = "+CapSlpRight90+"\n";
		    res += "CapSlpRotate180 = "+CapSlpRotate180+"\n";
		    res += "CharacterSet = "+CharacterSet+"\n";
		    res += "CharacterSetList = "+CharacterSetList+"\n";
		    res += "CoverOpen = "+CoverOpen+"\n";
		    res += "ErrorLevel = "+ErrorLevel+"\n";
		    res += "FlagWhenIdle = "+FlagWhenIdle+"\n";
		    res += "MapMode = "+MapMode+"\n";
		    res += "RecLineChars = "+RecLineChars+"\n";
		    res += "RecLineCharsList = "+RecLineCharsList+"\n";
		    res += "RecLineHeight = "+RecLineHeight+"\n";
		    res += "RecLineWidth = "+RecLineWidth+"\n";
		    res += "RecEmpty = "+RecEmpty+"\n";
		    res += "SlpLineChars = "+SlpLineChars+"\n";
		    res += "SlpLineCharsList = "+SlpLineCharsList+"\n";
		    res += "SlpLineHeight = "+SlpLineHeight+"\n";
		    res += "SlpLineSpacing = "+SlpLineSpacing+"\n";
		    res += "SlpEmpty = "+SlpEmpty+"\n";
		    res += "SlpPrintSide = "+SlpPrintSide+"\n";
			
			res += "Time taken: "+getTimeDifference(start)+"ms";
		}
		
		return res;
	} 
	
	/*
	 * This method demonstrates the use of printer methods supported by the TGCSMPOS API to 
	 * print a sample receipt. In this example, the receipt data is communicated to the printer 
	 * synchronously, having each method return only after the data is transmitted and processed by 
	 * the printer. 
	 * @param: logicalDeviceName: logical device name of the target.
	 * @effects: prints sample receipt on the device.
	 * @return: String result
	 */
	private String PrinterReceipt(String logicalDeviceName) {
		String res = "";
		
		TGCSMPOS4610Printer device = new TGCSMPOS4610Printer();
		
		String content = getTextFromAssets("sample_receipt.txt");
		Bitmap logo = getBitmapFromAsset(ctx, "toshiba-logo.png");
		
		java.util.Date start = new java.util.Date();
		try {
			// open the device
			device.open(logicalDeviceName);
			
			// claim the device for exclusive access; 3000ms timeout
			device.claim(3000);

			// enable the device
			device.setDeviceEnabled(true);
			
			// logo
			device.printBitmap(POS.PTR_S_RECEIPT, logo, POS.PTR_BM_ASIS, POS.PTR_BC_CENTER);
			
			// content
			device.printNormal(POS.PTR_S_RECEIPT, content);
			
			// barcode
			device.printBarCode(POS.PTR_S_RECEIPT, "0123456789", POS.PTR_BCS_Code39, 100, 2, POS.PTR_BC_CENTER, POS.PTR_BC_TEXT_ABOVE);
			
			if (autoCutPaper) device.cutPaper(100);
			
			// relinquish exclusive access to the device
			device.release();
			
			// close the device
			device.close();
		} catch (TGCSMPOSException mpos_e) {
			res += mpos_e.getMessage() + "\n";
		} catch (RuntimeException e) {
			res += e.getMessage() + "\n";
		} finally {
			try {
				// necessary clean up
				device.close();
			} catch(Exception e1) {
				// in the event open fails, close() will throw an exception
				// ignore
			}
		}
		
		
		res += "Time taken: "+getTimeDifference(start)+"ms";
		
		return res;
	}
	
	/*
	 * This method demonstrates the use of ESC sequences in printing.
	 * @param: logicalDeviceName: logical device name of the target.
	 * @effects: prints sample receipt on the device.
	 * @return: String result
	 */
	private String PrinterReceiptEscSequences(String logicalDeviceName) {
		String res = "";
		
		TGCSMPOS4610Printer device = new TGCSMPOS4610Printer();
		
		String content = getTextFromAssets("escape_sequences_full_eg.txt");
		
		java.util.Date start = new java.util.Date();
		try {
			// open the device
			device.open(logicalDeviceName);
			
			// claim the device for exclusive access; 3000ms timeout
			device.claim(3000);

			// enable the device
			device.setDeviceEnabled(true);
			
			// content
			device.printNormal(POS.PTR_S_RECEIPT, content);
			
			if (autoCutPaper) device.cutPaper(100);
			
			// relinquish exclusive access to the device
			device.release();
			
			// close the device
			device.close();
		} catch (TGCSMPOSException mpos_e) {
			res += mpos_e.getMessage() + "\n";
		} catch (RuntimeException e) {
			res += e.getMessage() + "\n";
		} finally {
			try {
				// necessary clean up
				device.close();
			} catch(Exception e1) {
				// in the event open fails, close() will throw an exception
				// ignore
			}
		}
		
		
		res += "Time taken: "+getTimeDifference(start)+"ms";
		
		return res;
	}
	
	/*
	 * This example demonstrates the use rotatePrint().
	 * @param: logicalDeviceName: logical device name of the target.
	 * @effects: prints sample receipt on the device.
	 * @return: String result
	 */
	private String PrinterReceiptRotatePrintSynchronous(String logicalDeviceName) {
		String res = "";
		
		TGCSMPOS4610Printer device = new TGCSMPOS4610Printer();
		
		String content = getTextFromAssets("sample_receipt_2.txt");
		
		java.util.Date start = new java.util.Date();
		try {
			// open the device
			device.open(logicalDeviceName);
			
			// claim the device for exclusive access; 3000ms timeout
			device.claim(3000);

			// enable the device
			device.setDeviceEnabled(true);
			
			// content
			device.rotatePrint(POS.PTR_S_RECEIPT, POS.PTR_RP_RIGHT90);
			device.printNormal(POS.PTR_S_RECEIPT, "PTR_RP_RIGHT90");
			device.printNormal(POS.PTR_S_RECEIPT, content);
			device.rotatePrint(POS.PTR_S_RECEIPT, POS.PTR_RP_NORMAL);
			device.rotatePrint(POS.PTR_S_RECEIPT, POS.PTR_RP_LEFT90);
			device.printNormal(POS.PTR_S_RECEIPT, "PTR_RP_LEFT90");
			device.printNormal(POS.PTR_S_RECEIPT, content);
			device.rotatePrint(POS.PTR_S_RECEIPT, POS.PTR_RP_NORMAL);
			device.rotatePrint(POS.PTR_S_RECEIPT, POS.PTR_RP_ROTATE180);
			device.printNormal(POS.PTR_S_RECEIPT, "PTR_RP_ROTATE180");
			device.printNormal(POS.PTR_S_RECEIPT, content);
			device.rotatePrint(POS.PTR_S_RECEIPT, POS.PTR_RP_NORMAL);
			device.printNormal(POS.PTR_S_RECEIPT, "PTR_RP_NORMAL");
			device.printNormal(POS.PTR_S_RECEIPT, content);
			
			if (autoCutPaper) device.cutPaper(100);
			
			// relinquish exclusive access to the device
			device.release();
			
			// close the device
			device.close();
		} catch (TGCSMPOSException mpos_e) {
			res += mpos_e.getMessage() + "\n";
		} catch (RuntimeException e) {
			res += e.getMessage() + "\n";
		} finally {
			try {
				// necessary clean up
				device.close();
			} catch(Exception e1) {
				// in the event open fails, close() will throw an exception
				// ignore
			}
		}
		
		
		res += "Time taken: "+getTimeDifference(start)+"ms";
		
		return res;
	}
	
	/*
	 * This example demonstrates the use rotatePrint() in conglomeration with 
	 * ESC Sequences. 
	 * @param: logicalDeviceName: logical device name of the target.
	 * @effects: prints sample receipt on the device.
	 * @return: String result
	 */
	private String PrinterReceiptRotateEscSequencePrintSynchronous(String logicalDeviceName) {
		String res = "";
		
		TGCSMPOS4610Printer device = new TGCSMPOS4610Printer();
		
		String content = getTextFromAssets("escape_sequence_eg.txt");
		
		java.util.Date start = new java.util.Date();
		try {
			// open the device
			device.open(logicalDeviceName);
			
			// claim the device for exclusive access; 3000ms timeout
			device.claim(3000);

			// enable the device
			device.setDeviceEnabled(true);
			
			// content
			device.rotatePrint(POS.PTR_S_RECEIPT, POS.PTR_RP_RIGHT90);
			device.printNormal(POS.PTR_S_RECEIPT, "PTR_RP_RIGHT90\n");
			device.printNormal(POS.PTR_S_RECEIPT, content);
			device.rotatePrint(POS.PTR_S_RECEIPT, POS.PTR_RP_NORMAL);
			device.rotatePrint(POS.PTR_S_RECEIPT, POS.PTR_RP_LEFT90);
			device.printNormal(POS.PTR_S_RECEIPT, "PTR_RP_LEFT90\n");
			device.printNormal(POS.PTR_S_RECEIPT, content);
			device.rotatePrint(POS.PTR_S_RECEIPT, POS.PTR_RP_NORMAL);
			device.rotatePrint(POS.PTR_S_RECEIPT, POS.PTR_RP_ROTATE180);
			device.printNormal(POS.PTR_S_RECEIPT, "PTR_RP_ROTATE180\n");
			device.printNormal(POS.PTR_S_RECEIPT, content);
			device.rotatePrint(POS.PTR_S_RECEIPT, POS.PTR_RP_NORMAL);
			device.printNormal(POS.PTR_S_RECEIPT, "PTR_RP_NORMAL\n");
			device.printNormal(POS.PTR_S_RECEIPT, content);
			
			if (autoCutPaper) device.cutPaper(100);
			
			// relinquish exclusive access to the device
			device.release();
			
			// close the device
			device.close();
		} catch (TGCSMPOSException mpos_e) {
			res += mpos_e.getMessage() + "\n";
		} catch (RuntimeException e) {
			res += e.getMessage() + "\n";
		} finally {
			try {
				// necessary clean up
				device.close();
			} catch(Exception e1) {
				// in the event open fails, close() will throw an exception
				// ignore
			}
		}
		
		
		res += "Time taken: "+getTimeDifference(start)+"ms";
		
		return res;
	}
	
	/*
	 * This method demonstrates the use of printer methods supported by the TGCSMPOS API 
	 * to print a receipt on the printer via transaction mode. This test method first initiates the 
	 * transaction mode on the printer to hold the buffer, transmits the data to the printer and 
	 * finally releasing the buffer to perform the print.  
	 * @param: logicalDeviceName: logical device name of the target.
	 * @effects: prints receipt on the device
	 * @return: String result
	 */
	private String PrinterReceiptTransaction(String logicalDeviceName) {
		String res = "";
		
		TGCSMPOS4610Printer device = new TGCSMPOS4610Printer();
		
		String content = getTextFromAssets("sample_receipt.txt");
		Bitmap logo = getBitmapFromAsset(ctx, "toshiba-logo.png");
		
		java.util.Date start = new java.util.Date();
		try {
			// open the device
			device.open(logicalDeviceName);
			
			// claim the device for exclusive access; 3000ms timeout
			device.claim(3000);
			
			// enable the device
			device.setDeviceEnabled(true);
			
			// initiate transaction
			device.transactionPrint(POS.PTR_S_RECEIPT, POS.PTR_TP_TRANSACTION);
			
			// logo
			device.printBitmap(POS.PTR_S_RECEIPT, logo, POS.PTR_BM_ASIS, POS.PTR_BC_CENTER);
			
			// content
			device.printNormal(POS.PTR_S_RECEIPT, content);
			
			// barcode
			device.printBarCode(POS.PTR_S_RECEIPT, "0123456789", POS.PTR_BCS_Code39, 100, 2, POS.PTR_BC_CENTER, POS.PTR_BC_TEXT_ABOVE);
			
			if (autoCutPaper) device.cutPaper(100);
			
			// commit transaction
			device.transactionPrint(POS.PTR_S_RECEIPT, POS.PTR_TP_NORMAL);
			
			// relinquish exclusive access to the device
			device.release();
			
			// close the device
			device.close();
		} catch (TGCSMPOSException mpos_e) {
			res += mpos_e.getMessage() + "\n";
		} catch (RuntimeException e) {
			res += e.getMessage() + "\n";
		} finally {
			try {
				// necessary clean up
				device.close();
			} catch(Exception e1) {
				// in the event open fails, close() will throw an exception
				// ignore
			}
		}
		
		
		res += "Time taken: "+getTimeDifference(start)+"ms";
		
		return res;
	}

	/*
	 * This method demonstrates the use of printer methods supported by the TGCSMPOS API 
	 * to print a receipt on the printer asynchronously. In asynchronous mode, it is important 
	 * to wait for the queued operations to complete to avoid premature release of resources. 
	 * In this example, this method demonstrate the use of waiting for the StatusUpdateEvent
	 * before allowing the main thread to proceed.  
	 * @param: logicalDeviceName: logical device name of the target.
	 * @effects: prints receipt on the device
	 * @return: String result
	 */
	private String PrinterReceiptAsyncModeMethod1(String logicalDeviceName) {
		String res = "";
		
		TGCSMPOS4610Printer device = new TGCSMPOS4610Printer();
		
		final Object lock = new Object();
		
		/*====================================================
		 * Asynchronous method calling METHOD #1
		 * 
		 * The strategy is to wait for the virtual system to return
		 * back to S_IDLE state after it finishes processing 
		 * the queue of asynchronous method calls. The virtual
		 * device will enter the S_BUSY state when processing
		 * the queue and S_IDLE state when it is finished. 
		 ====================================================*/
		
		device.addPOSEventListener(new POSEventHandler() {

			/*====================================================
			 * Asynchronous method calling METHOD #1
			 * 
			 * Wait for StatusUpdateEvent with the status PTR_SUE_IDLE
			 * (1001) value. Upon receiving it, use the lock to 
			 * notify waiting processes/threads to proceed.
			 ====================================================*/
			@Override
			public void handleStatusUpdateEvent(final POSStatusUpdateEvent event) {
				
				((Activity)ctx).runOnUiThread(new Runnable() {

					@Override
					public void run() {
						Toast.makeText(
								getApplicationContext(),
								event.getMessage(), Toast.LENGTH_SHORT)
								.show();
						
						synchronized(lock) {
							lock.notifyAll();
						}
						
					}
					
				});
			}

			@Override
			public void handleDataEvent(POSDataEvent event) {
				// TODO Auto-generated method stub
				
			}

			@Override
			public void handleErrorEvent(final POSErrorEvent event) {
				Log.d("TGCS Sample App", "POSErrorEvent("+event.getErrorCode()+","+event.getErrorCodeExtended()+","+event.getErrorLocus()+","+event.getErrorResponse()+","+event.getMessage()+")");
				
				// if an error is encountered, clear output cache and terminate the job.
				event.setErrorResponse(POS.POS_ER_CLEAR);
				
				((Activity)ctx).runOnUiThread(new Runnable() {

					@Override
					public void run() {
						Toast.makeText(
								getApplicationContext(),
								ctx.getString(
										R.string.template_error_event,
										event.getErrorCode(),
										event.getErrorCodeExtended(),
										event.getErrorLocus(),
										event.getErrorResponse()
										), Toast.LENGTH_SHORT)
								.show();
						
						synchronized(lock) {
							lock.notifyAll();
						}
					}
					
				});
			}

			@Override
			public void handleEvent(POSEvent event) {
				// TODO Auto-generated method stub
				
			}

			@Override
			public void handleOutputCompleteEvent(final POSOutputCompleteEvent arg0) {
				Log.d("TGCS Sample App", "POSOutputCompleteEvent("+arg0.getOutputID()+")");
				
				((Activity)ctx).runOnUiThread(new Runnable() {

					@Override
					public void run() {
						Toast.makeText(
								getApplicationContext(),
								ctx.getString(
										R.string.template_output_complete_event,
										arg0.getOutputID()
										), Toast.LENGTH_LONG)
								.show();
					}
					
				});
			}
			
		});
		
		String content = getTextFromAssets("sample_receipt.txt");
		Bitmap logo = getBitmapFromAsset(ctx, "toshiba-logo.png");
		
		java.util.Date start = new java.util.Date();
		try {
			// open the device
			device.open(logicalDeviceName);
			
			// claim the device for exclusive access; 3000ms timeout
			device.claim(3000);

			// enable the device
			device.setDeviceEnabled(true);		
			
			// turn on AsyncMode
			device.setAsyncMode(true);
			
			/*====================================================
			 * Asynchronous method calling METHOD #1
			 * 
			 * This flag must be set to true for METHOD #1 to work. 
			 * This tells the system to fire POSStatusUpdateEvents
			 * with status (POS_SUE_IDLE) when the system enters 
			 * the S_IDLE state.
			 ====================================================*/
			device.setFlagWhenIdle(true);
			
			// logo
			device.printBitmap(POS.PTR_S_RECEIPT, logo, 576, POS.PTR_BC_CENTER);
			
			// content
			device.printNormal(POS.PTR_S_RECEIPT, content);
			
			// barcode
			device.printBarCode(POS.PTR_S_RECEIPT, "0123456789", POS.PTR_BCS_Code39, 100, 2, POS.PTR_BC_CENTER, POS.PTR_BC_TEXT_ABOVE);
			
			if (autoCutPaper) device.cutPaper(100);
			
			/*====================================================
			 * Asynchronous method calling METHOD #1
			 * 
			 * Wait for StatusUpdateEvent to arrive.
			 ====================================================*/
			synchronized(lock) {
				try {
					lock.wait(8000);
				} catch(InterruptedException e) {
					e.printStackTrace();
				}
 			}
			
			// relinquish exclusive access to the device
			device.release();
			
			// close the device
			device.close();
		} catch (TGCSMPOSException mpos_e) {
			res += mpos_e.getMessage() + "\n";
		} catch (RuntimeException e) {
			res += e.getMessage() + "\n";
		} finally {
			try {
				// necessary clean up
				device.close();
			} catch(Exception e1) {
				// in the event open fails, close() will throw an exception
				// ignore
			}
		}
		
		
		res += "Time taken: "+getTimeDifference(start)+"ms";
		
		return res;
	}
	
	/*
	 * This method demonstrates the use of printer methods supported by the TGCSMPOS API 
	 * to print a receipt on the printer asynchronously. In asynchronous mode, it is important 
	 * to wait for the queued operations to complete to avoid premature release of resources. 
	 * In this example, this method demonstrate the use of waiting for the OutputCompleteEvent
	 * before allowing the main thread to proceed.  
	 * @param: logicalDeviceName: logical device name of the target.
	 * @effects: prints receipt on the device
	 * @return: String result
	 */
	private String PrinterReceiptAsyncModeMethod2(String logicalDeviceName) {
		String res = "";
		
		TGCSMPOS4610Printer device = new TGCSMPOS4610Printer();
		
		final Object lock = new Object();
		
		/*====================================================
		 * Asynchronous method calling METHOD #2
		 * 
		 * Every asynchronous request is afforded an OutputID.
		 * The Strategy is to wait for the last OutputID to 
		 * know that the asynchronous method calls have been 
		 * processed and executed.
		 ====================================================*/
		
		device.addPOSEventListener(new POSEventHandler() {

			/*====================================================
			 * Asynchronous method calling METHOD #2
			 * 
			 * Wait for OutputCompleteEvent with the OutputID attached
			 * to the event. When the OutputID matches the expectedOutputID
			 * the device is ready to proceed.
			 ====================================================*/
			@Override
			public void handleOutputCompleteEvent(final POSOutputCompleteEvent event) {
				
				if (event.getOutputID() == expectedOutputID) {
					((Activity)ctx).runOnUiThread(new Runnable() {

						@Override
						public void run() {
							Toast.makeText(
									getApplicationContext(),
									ctx.getString(
											R.string.template_outputID_received,
											event.getOutputID()), Toast.LENGTH_LONG)
									.show();
							
							synchronized(lock) {
								lock.notifyAll();
							}
							
						}
						
					});
				}
			}

			@Override
			public void handleDataEvent(POSDataEvent arg0) {
				// TODO Auto-generated method stub
				
			}

			@Override
			public void handleErrorEvent(final POSErrorEvent event) {
				Log.d("TGCS Sample App", "POSErrorEvent("+event.getErrorCode()+","+event.getErrorCodeExtended()+","+event.getErrorLocus()+","+event.getErrorResponse()+")");
				
				// if an error is encountered, clear output cache and terminate the job.
				event.setErrorResponse(POS.POS_ER_CLEAR);
				
				((Activity)ctx).runOnUiThread(new Runnable() {

					@Override
					public void run() {
						Toast.makeText(
								getApplicationContext(),
								ctx.getString(
										R.string.template_error_event,
										event.getErrorCode(),
										event.getErrorCodeExtended(),
										event.getErrorLocus(),
										event.getErrorResponse()
										), Toast.LENGTH_SHORT)
								.show();
						
						synchronized(lock) {
							lock.notifyAll();
						}
					}
					
				});
			}

			@Override
			public void handleEvent(POSEvent arg0) {
				// TODO Auto-generated method stub
				
			}

			@Override
			public void handleStatusUpdateEvent(POSStatusUpdateEvent arg0) {
				// TODO Auto-generated method stub
				
			}
			
		});
		
		String content = getTextFromAssets("sample_receipt.txt");
		Bitmap logo = getBitmapFromAsset(ctx, "toshiba-logo.png");
		
		/*====================================================
		 * Asynchronous method calling METHOD #2
		 * 
		 * Reset the expectedOutputID.
		 ====================================================*/
		expectedOutputID = 0;
		
		java.util.Date start = new java.util.Date();
		try {
			// open the device
			device.open(logicalDeviceName);
			
			// claim the device for exclusive access; 3000ms timeout
			device.claim(3000);

			// enable the device
			device.setDeviceEnabled(true);		
			
			// turn on AsyncMode
			device.setAsyncMode(true);
			
			// logo
			device.printBitmap(POS.PTR_S_RECEIPT, logo, 576, POS.PTR_BC_CENTER);
			
			// content
			device.printNormal(POS.PTR_S_RECEIPT, content);
			
			// barcode
			device.printBarCode(POS.PTR_S_RECEIPT, "0123456789", POS.PTR_BCS_Code39, 100, 2, POS.PTR_BC_CENTER, POS.PTR_BC_TEXT_ABOVE);
			
			if (autoCutPaper) device.cutPaper(100);
			
			/*====================================================
			 * Asynchronous method calling METHOD #2
			 * 
			 * Get the latest OutputID. 
			 ====================================================*/
			expectedOutputID = device.getOutputID();
			
			/*====================================================
			 * Asynchronous method calling METHOD #2
			 * 
			 * Wait for OutputCompleteEvent to arrive.
			 ====================================================*/
			synchronized(lock) {
				try {
					lock.wait(30000);
				} catch(InterruptedException e) {
					e.printStackTrace();
				}
 			}
			
			// relinquish exclusive access to the device
			device.release();
			
			// close the device
			device.close();
		} catch (TGCSMPOSException mpos_e) {
			res += mpos_e.getMessage() + "\n";
		} catch (RuntimeException e) {
			res += e.getMessage() + "\n";
		} finally {
			try {
				// necessary clean up
				device.close();
			} catch(Exception e1) {
				// in the event open fails, close() will throw an exception
				// ignore
			}
		}
		
		
		res += "Time taken: "+getTimeDifference(start)+"ms";
		
		return res;
	}

	/*
	 * This method demonstrates the use of printer methods supported by the TGCSMPOS API 
	 * to print examples of the supported RecLineChars supported by the printer.  
	 * @param: logicalDeviceName: logical device name of the target.
	 * @effects: prints textual data on the device
	 * @return: String result
	 */
	private String PrinterReceiptLineChars(String logicalDeviceName) {
		String res = "";
		
		TGCSMPOS4610Printer device = new TGCSMPOS4610Printer();
	
		String content = getTextFromAssets("ruler.txt");
		
		java.util.Date start = new java.util.Date();
		try {
			// open the device
			device.open(logicalDeviceName);
			
			// claim the device for exclusive access; 3000ms timeout
			device.claim(3000);
		
			// enable the device
			device.setDeviceEnabled(true);		
			
			// hold the buffer
			device.transactionPrint(POS.PTR_S_RECEIPT, POS.PTR_TP_TRANSACTION);
			
			for (int i = 72; i >= 22; i--) {
				device.setRecLineChars(i);
				device.printNormal(POS.PTR_S_RECEIPT, "RecLineChars: " + i + "("+device.getRecLineChars()+")" + (device.getRecLineChars() < i ? "--- \\x1b|bCCHECK" :""));
				device.printNormal(POS.PTR_S_RECEIPT, buildRuler(i) + "\n");
			}

			// cut paper
			if (autoCutPaper) device.cutPaper(100);
			
			// release the buffer
			device.transactionPrint(POS.PTR_S_RECEIPT, POS.PTR_TP_NORMAL);
			
			// relinquish exclusive access to the device
			device.release();
			
			// close the device
			device.close();
		} catch (TGCSMPOSException mpos_e) {
			res += mpos_e.getMessage() + "\n";
		} catch (RuntimeException e) {
			res += e.getMessage() + "\n";
		} finally {
			try {
				// necessary clean up
				device.close();
			} catch(Exception e1) {
				// in the event open fails, close() will throw an exception
				// ignore
			}
		}
		
		
		res += "Time taken: "+getTimeDifference(start)+"ms";
		
		return res;
	}

	/*
	 * This method demonstrates the use of printer methods supported by the TGCSMPOS API 
	 * to cut the receipt paper on the CR station
	 * @param: logicalDeviceName: logical device name of the target.
	 * @effects: Cuts the receipt paper
	 * @return: String result
	 */
	private String PrinterReceiptCutPaper(String logicalDeviceName) {
		String res = "";
		
		TGCSMPOS4610Printer device = new TGCSMPOS4610Printer();
		
		java.util.Date start = new java.util.Date();
		try {
			// open the device
			device.open(logicalDeviceName);
			
			// claim the device for exclusive access; 3000ms timeout
			device.claim(3000);
			
			// enable the device
			device.setDeviceEnabled(true);			

			// cut paper
			device.cutPaper(100);
			
			// relinquish exclusive access to the device
			device.release();
			
			// close the device
			device.close();
		} catch (TGCSMPOSException mpos_e) {
			res += mpos_e.getMessage() + "\n";
		} catch (RuntimeException e) {
			res += e.getMessage() + "\n";
		} finally {
			try {
				// necessary clean up
				device.close();
			} catch(Exception e1) {
				// in the event open fails, close() will throw an exception
				// ignore
			}
		}
		
		
		res += "Time taken: "+getTimeDifference(start)+"ms";
		
		return res;
	}

	/*
	 * This method demonstrates the use of printer methods supported by the TGCSMPOS API 
	 * to print a receipt on the Slip Station of the printer.
	 * @param: logicalDeviceName: logical device name of the target.
	 * @effects: prints receipt on the Slip Station
	 * @return: String result
	 */
	private String PrinterDocument(String logicalDeviceName) {
		String res = "";
		
		TGCSMPOS4610Printer device = new TGCSMPOS4610Printer();
		
		String content = getTextFromAssets("sample_receipt.txt");
		
		Object lock = new Object();
		bDichotDecision = false;
		showDichotomousDialog("Document Example", "This example demonstrates the SDK's capability to print on the Mumford Printer slip station. Please have a document ready. Tap OK to proceed.", lock);
		waitForResponse(lock);
		
		if (!bDichotDecision) {
			return "User Cancelled";
		}
		
		java.util.Date start = new java.util.Date();
		try {
			// open the device
			device.open(logicalDeviceName);
			
			// claim the device for exclusive access; 3000ms timeout
			device.claim(3000);
			
			// enable the device
			device.setDeviceEnabled(true);			
			
			if (device.getSlpEmpty()) {
				showInstructiveDialog("Document Example", "Please place the document in the slip station bay. Tap OK to continue.", lock);
				waitForResponse(lock);
			}
			
			// prepare receipt of document
			device.beginInsertion(30000);
			
			// end insertion
			device.endInsertion();
			
			// content
			device.printNormal(POS.PTR_S_SLIP, content);
			
			// barcode
			device.printBarCode(POS.PTR_S_SLIP, "0123456789", POS.PTR_BCS_Code39, 100, 2, POS.PTR_BC_CENTER, POS.PTR_BC_TEXT_ABOVE);
			
			if (autoCutPaper) device.cutPaper(100);
			
			// initiate removal
			device.beginRemoval(15000);
			
			// end removal
			device.endRemoval();
			
			// relinquish exclusive access to the device
			device.release();
			
			// close the device
			device.close();
		} catch (TGCSMPOSException mpos_e) {
			res += mpos_e.getMessage() + "\n";
		} catch (RuntimeException e) {
			res += e.getMessage() + "\n";
		} finally {
			try {
				// necessary clean up
				device.close();
			} catch(Exception e1) {
				// in the event open fails, close() will throw an exception
				// ignore
			}
		}
		
		
		res += "Time taken: "+getTimeDifference(start)+"ms";
		
		return res;
	}

	/*
	 * This method demonstrates the use of MICR methods supported by the TGCSMPOS API 
	 * to accept a check document, read and parse the MICR information and eject the check document.
	 * For MICR reads, being event driven it is necessary to set DataEventEnabled to true before performing 
	 * the read and waiting for POSDataEvent before obtaining the data from the API cache. 
	 * @param: logicalDeviceName: logical device name of the target.
	 * @effects: perform MICR read on the device
	 * @return: String result
	 */
	private String micrExample(String logicalDeviceName) {
		String res = "";
		
		TGCSMPOS4610MICR device = new TGCSMPOS4610MICR();
		
		// setup lock for event-driven device
		final Object lock = new Object();
		
		bDichotDecision = false;
		showDichotomousDialog("MICR Example", "This example demonstrates the use of the SDK to obtain MICR information from the Mumford Printer. Please prepare a test check. Tap OK to continue.", lock);
		waitForResponse(lock);
		
		if (!bDichotDecision) {
			return "User Cancelled";
		}
		
		device.addPOSEventListener(new POSEventHandler() {

			@Override
			public void handleDataEvent(final POSDataEvent event) {
				
				((Activity)ctx).runOnUiThread(new Runnable() {

					@Override
					public void run() {
						Toast.makeText(getApplicationContext(), ctx.getString(R.string.template_data_event, event.getStatus()), Toast.LENGTH_SHORT).show();
						
						synchronized(lock) {
							lock.notifyAll();
						}
					}
					
				});
			}

			@Override
			public void handleErrorEvent(POSErrorEvent arg0) {
				// TODO Auto-generated method stub
				
			}

			@Override
			public void handleEvent(POSEvent arg0) {
				// TODO Auto-generated method stub
				
			}

			@Override
			public void handleOutputCompleteEvent(POSOutputCompleteEvent arg0) {
				// TODO Auto-generated method stub
				
			}

			@Override
			public void handleStatusUpdateEvent(POSStatusUpdateEvent arg0) {
				// TODO Auto-generated method stub
				
			}
			
		});
		
		String AccountNumber = "";
	    String BankNumber = "";
	    int CheckType = 0;
	    int CountryCode = 0;
	    String EPC = "";
	    String RawData = "";
	    String SerialNumber = "";
	    String TransitNumber = "";
		
		java.util.Date start = new java.util.Date();
		try {
			// open the device
			device.open(logicalDeviceName);
			
			// claim the device for exclusive access; 3000ms timeout
			device.claim(3000);
			
			// enable the device
			device.setDeviceEnabled(true);		
			
			// enable data events
			device.setDataEventEnabled(true); 
			
			{
				showInstructiveDialog("MICR Example", "Please place the check in the slip bay of the printer. Tap OK to continue.", lock);
				waitForResponse(lock);
			}
			
			// prepare receipt of document
			device.beginInsertion(30000);
			
			// end insertion
			device.endInsertion();
			
			// lock it up
			synchronized(lock) {
				try {
					// wait for event
					// TODO change this to a time out wait.
					lock.wait(30000);
					// TODO timed out message
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
				
				// Event received, data ready.
				AccountNumber = device.getAccountNumber();
			    BankNumber = device.getBankNumber();
			    CheckType = device.getCheckType();
			    CountryCode = device.getCountryCode();
			    EPC = device.getEPC();
			    RawData = device.getRawData();
			    SerialNumber = device.getSerialNumber();
			    TransitNumber = device.getTransitNumber();
			}
			
			// initiate removal
			device.beginRemoval(15000);
			
			// end removal
			device.endRemoval();
			
			// relinquish exclusive access to the device
			device.release();
			
			// close the device
			device.close();
		} catch (TGCSMPOSException mpos_e) {
			res += mpos_e.getMessage() + "\n";
		} catch (RuntimeException e) {
			res += e.getMessage() + "\n";
		} finally {
			try {
				// necessary clean up
				device.close();
			} catch(Exception e1) {
				// in the event open fails, close() will throw an exception
				// ignore
			}
		}
		
		
		if (res.isEmpty()) {
			res += "AccountNumber = "+AccountNumber+"\n";
		    res += "BankNumber = "+BankNumber+"\n";
		    res += "CheckType = "+CheckType+"\n";
		    res += "CountryCode = "+CountryCode+"\n";
		    res += "EPC = "+EPC+"\n";
		    res += "RawData = "+RawData+"\n";
		    res += "SerialNumber = "+SerialNumber+"\n";
		    res += "TransitNumber = "+TransitNumber+"\n";
		}
		res += "Time taken: "+getTimeDifference(start)+"ms";
		
		return res;
	}

	/*
	 * This method demonstrates the use of cash drawer methods supported by the TGCSMPOS API 
	 * to open the cash drawer and actively waiting for it to close. This method demonstrates the 
	 * use of waitForDrawerClose() method.
	 * @param: logicalDeviceName: logical device name of the target.
	 * @effects: opens the cash drawer
	 * @return: String result
	 */
	private String cashDrawerExample(String logicalDeviceName) {
		String res = "";
		
		TGCSMPOS4610CashDrawer device = new TGCSMPOS4610CashDrawer();
		
		boolean DrawerOpenedBefore = false;
		boolean DrawerOpenedAfter = false;
		
		java.util.Date start = new java.util.Date();
		try {
			// open the device
			device.open(logicalDeviceName);
			
			// claim the device for exclusive access; 3000ms timeout
			device.claim(3000);
			
			// enable the device
			device.setDeviceEnabled(true);		
			
			// obtain the status before the command is executed.
			DrawerOpenedBefore = device.getDrawerOpened();
			
			// open the cash drawer
			device.openDrawer();
			
			// obtain the status after the command is executed
			DrawerOpenedAfter = device.getDrawerOpened();
			
			// wait for drawer to close. this method will return when the drawer is closed.
			device.waitForDrawerClose(3000, 47, 200, 800);
			
			// relinquish exclusive access to the device
			device.release();
			
			// close the device
			device.close();
		} catch (TGCSMPOSException mpos_e) {
			res += mpos_e.getMessage() + "\n";
		} catch (RuntimeException e) {
			res += e.getMessage() + "\n";
		} finally {
			try {
				// necessary clean up
				device.close();
			} catch(Exception e1) {
				// in the event open fails, close() will throw an exception
				// ignore
			}
		}
		
		if (res.isEmpty()) {
			res += "DrawerOpenedBefore = " + DrawerOpenedBefore + "\n";
			res += "DrawerOpenedAfter = " + DrawerOpenedAfter + "\n";
		}
		res += "Time taken: "+getTimeDifference(start)+"ms";
		
		return res;
	}
	
	/*
	 * This method demonstrates the use of global methods supported by the TGCSMPOS API 
	 * to load a configuration file to the TGCSMPOS system.  This method prompts the user
	 * to select the efg file and laoding it into the system.
	 * @param: logicalDeviceName: logical device name of the target.
	 * @effects: loads efg file to system
	 * @return: void
	 */
	private void loadConfiguredDevices() {
		ConfiguredDevice list[] = TGCSMPOSDevice.getDevices();
		String msg = "";
		if (list.length > 0) {
			for (int i = 0; i < list.length; i++) {
				msg += list[i].getLogicalName() + "\n";
				msg += list[i].getDeviceAddress() + "\n";
				msg += list[i].getDeviceName() + "\n";
				msg += list[i].getDescription() + "\n";
				msg += "UDP Port: " + list[i].getDeviceUdpPort() + "" + "\n";
				msg += "TCP Port: " + list[i].getDeviceTcpPort() + "" + "\n";
				msg += "\n";
			}
		} else {
			msg = "No Devices Configured.";
		}
		
		new AlertDialog.Builder(ctx)
		.setTitle(getResources().getString(R.string.title_loadConfiguration))
		.setMessage(msg)
		.setPositiveButton(ctx.getResources().getString(R.string.title_ok), new DialogInterface.OnClickListener() {
			
			@Override
			public void onClick(DialogInterface dialog, int which) {
				dialog.dismiss();
			}
		})
		.show();
	}
	
	private String buildRuler(int x) {
		StringBuilder sb = new StringBuilder();
		int counter = 0;

		for (int i = 0; i < x; i++) {
			int tens = ((i/10) * 10) + 10;

			switch(counter) {
			case 0: sb.append('<'); break;
			case 9: sb.append('>'); break;
			case 4: {
				sb.append((int)tens/10);
			} break;
			case 5: {
				sb.append((int)tens%10);
			} break;
			default: sb.append('-'); break;
			}

			counter = (counter+1)%10;
		}


		return sb.toString();
	}
	
	/*
	 * This method has been left intentionally for developers to test the SDK prior to implementing
	 * the SDK in their code. Feel free to modify this example to suit your application needs. 
	 * @param: logicalDeviceName: logical device name of the target.
	 * @effects: performs tests on device
	 * @return: String result
	 */
	private String testGround(String logicalDeviceName) {
		String res = "";
		
		// instantiate the device instance
		final TGCSMPOS4610Printer device = new TGCSMPOS4610Printer();
		
		final Object lock = new Object();
		
		POSEventHandler eventHandler = new POSEventHandler() {

			@Override
			public void handleDataEvent(POSDataEvent event) {
				// TODO Auto-generated method stub
				
			}

			@Override
			public void handleErrorEvent(final POSErrorEvent event) {
				Log.d("Sample App", "ErrorEvent("+event.getErrorCode()+","+event.getErrorCodeExtended()+","+event.getErrorLocus()+","+event.getErrorResponse()+","+event.getMessage()+")");
				
				event.setErrorResponse(POS.POS_ER_CLEAR);
				
				synchronized(lock) {
					lock.notifyAll();
				}
				
				
			}

			@Override
			public void handleEvent(POSEvent event) {
				// TODO Auto-generated method stub
				
			}

			@Override
			public void handleOutputCompleteEvent(POSOutputCompleteEvent event) {
				Log.d("Sample App", "OutputCompleteEvent("+event.getOutputID()+")");
			}

			@Override
			public void handleStatusUpdateEvent(POSStatusUpdateEvent event) {
				Log.d("Sample App", "StatusUpdateEvent received("+event.getStatus()+")");
				synchronized(lock) {
					lock.notifyAll();
				}
				
			}
			
		};
		
		java.util.Date start = new java.util.Date();
		try {
			// open the device
			device.open(logicalDeviceName);
			
			// claim
			device.claim(3000);
			
			// enable
			device.setDeviceEnabled(true);
			
			// For AsyncMode, uncomment the following...
			//device.addPOSEventListener(eventHandler);
			//device.setAsyncMode(true);
			
			{
				// Your test case goes here.
			}
			
			// release
			device.release();
			
			// close
			device.close();
			
		} catch (TGCSMPOSException mpos_e) {
			Log.e("Sample App", "TGCSMPOSException: "+mpos_e.getMessage());
			res += mpos_e.getMessage() + "\n";
			try {
				// necessary clean up
				device.close();
			} catch(Exception e1) {
				// in the event open fails, close() will throw an exception
				// ignore
			}
		} catch (RuntimeException e) {
			Log.e("Sample App", "RuntimeException: "+e.getMessage());
			res += e.getMessage() + "\n";
			try {
				// necessary clean up
				device.close();
			} catch(Exception e1) {
				// in the event open fails, close() will throw an exception
				// ignore
			}
		}
		
		
		if (res.isEmpty()) {
			res += "Time taken: "+getTimeDifference(start)+"ms";
		}
		
		return res;
	}

	////////////////////////////////////////////////////////
	// Auxiliary methods
	////////////////////////////////////////////////////////
	
	
	
	private void showInformativeDialog(final String title, final String message) {
		runOnUiThread(new Runnable() {

			@Override
			public void run() {
				new AlertDialog.Builder(ctx)
				.setTitle(title)
				.setMessage(message)
				.setCancelable(false)
				.setPositiveButton("OK", new DialogInterface.OnClickListener() {
					
					@Override
					public void onClick(DialogInterface dialog, int id) {
						dialog.dismiss();
					}
				}).create().show();
				
			}
			
		});
	}
	
	private void showInstructiveDialog(final String title, final String message, final Object lock) {
		runOnUiThread(new Runnable() {

			@Override
			public void run() {
				new AlertDialog.Builder(ctx)
				.setTitle(title)
				.setMessage(message)
				.setCancelable(false)
				.setPositiveButton("OK", new DialogInterface.OnClickListener() {
					
					@Override
					public void onClick(DialogInterface dialog, int id) {
						dialog.dismiss();
						synchronized(lock) {
							lock.notifyAll();
						}
					}
				}).create().show();
				
			}
			
		});
	}
	
	private boolean bDichotDecision = false;
	
	private void showDichotomousDialog(final String title, final String message, final Object lock) {
		runOnUiThread(new Runnable() {

			@Override
			public void run() {
				new AlertDialog.Builder(ctx)
				.setTitle(title)
				.setMessage(message)
				.setCancelable(false)
				.setPositiveButton("OK", new DialogInterface.OnClickListener() {
					
					@Override
					public void onClick(DialogInterface dialog, int id) {
						bDichotDecision = true;
						dialog.dismiss();
						synchronized(lock) {
							lock.notifyAll();
						}
					}
				})
				.setNegativeButton("Cance", new DialogInterface.OnClickListener() {
					
					@Override
					public void onClick(DialogInterface dialog, int which) {
						bDichotDecision = false;
						dialog.dismiss();
						synchronized(lock) {
							lock.notifyAll();
						}
					}
				}).create().show();
				
			}
			
		});
	}
	
	private void waitForResponse(Object lock) {
		synchronized(lock) {
			try {
				lock.wait();
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	/*
	 * Read textual contents from asset files.
	 * @param: filename: file to read
	 * @return: String result
	 */
	private String getTextFromAssets(String filename) {
		AssetManager assetManager = getResources().getAssets();
		InputStream inputStream = null;
		String res = "";
		
		try {
			inputStream = assetManager.open(filename);
			
			if (inputStream != null) {
				InputStreamReader is = new InputStreamReader(inputStream);
				StringBuilder sb = new StringBuilder();
				BufferedReader br = new BufferedReader(is);
				String read = br.readLine();
				
				while (read != null) {
					sb.append(read);
					sb.append("\n");
					read = br.readLine();
				}
				
				res = sb.toString();
				
				inputStream.close();
			}
			
		} catch(IOException e) {
			Log.e("TGCSMPOS Sample", e.getMessage());
			e.printStackTrace();
		}
		
		return res;
	}

	/**
	 * Obtain bitmap data from assets.
	 * @param: context: contextual information
	 * @param: strName: file name
	 * @return: Bitmap 
	 */
	private Bitmap getBitmapFromAsset(Context context, String strName) {
	    AssetManager assetManager = context.getAssets();

	    InputStream istr;
	    Bitmap bitmap = null;
	    try {
	        istr = assetManager.open(strName);
	        bitmap = BitmapFactory.decodeStream(istr);
	    } catch (IOException e) {
	        return null;
	    }

	    return bitmap;
	}
	
	/**
	 * Convert a string of numbers separated by a comma(,) into an integer list. 
	 * @param: src: source string
	 * @return: Integer array
	 */
	private Integer[] getIntListFromString(String src) {
		
		String s[] = src.split(",");
		Integer[] list = new Integer[s.length];
		
		
		for (int i = 0; i < s.length; i++) {
			list[i] = Integer.valueOf(s[i]);
		}
		
		
		return list;
	}
	
	/**
	 * Return the time difference in ms from a given start time.
	 * @param start the start time
	 * @return time difference between now and the start time in ms.
	 */
	
	private long getTimeDifference(java.util.Date start) {
		return ((new java.util.Date()).getTime() - start.getTime());
	}
}
